# Generated by Django 4.2.1 on 2023-07-18 14:47

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("power_query", "0001_migration_squashed_0014_migration"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dataset",
            name="description",
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name="report",
            name="limit_access_to",
            field=models.ManyToManyField(
                blank=True, related_name="+", to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.AlterField(
            model_name="reportdocument",
            name="limit_access_to",
            field=models.ManyToManyField(
                blank=True, related_name="+", to=settings.AUTH_USER_MODEL
            ),
        ),
    ]
