* {
  font-family: Verdana;
}

table {
  width: 100%;

  tr td {
    padding: 5px;
  }
}

.title {
  font-size: 48px;
  font-weight: bold;
}

ul.breadcrumbs {
  li {
    display: inline;
    //float: left;
    list-style: none;
    padding: 0 2px;
  }

  li + li:before {
    color: black;
    content: "/\00a0";
  }
}

td.number {
  text-align: right;
}
